import java.applet.*;
import java.awt.*;
import java.net.*;
import java.io.*;
import java.util.*;
// Classes created by Resource Wizard for interface.
import DialogLayout;
import ConnectDlgRes;
import MainMenu;
import MainWinRes;
import NoteRes;
import ViewMessageRes;
import MessageBox;

public class EX18C extends Applet implements MailRetrieverClient
{
    MainWinRes mainWin = null;      // Main window resource.
    ConnectData connectData = null; // Used to exchange data with connect dialog.
    MailRetriever retriever = null; // Responsible for retrieving messages.
    Vector messages = new Vector(); // Holds retrieved messages.

    public EX18C()
    {
        connectData = new ConnectData();
    }

    public static void main(String args[])
    {   // Called when run as an application.
        EX18C applet = new EX18C();
        // Setup frame and main window.
        Ex18cApplicationFrame frame = new Ex18cApplicationFrame(applet);
        applet.mainWin = frame.resource;
        applet.mainWin.IDC_STATUS.setText("");
        applet.mainWin.IDC_VIEWMESSAGE.disable();
        applet.start();
    }

    public void init()
    {   // Running as an applet.
        mainWin = new MainWinRes(this);
        mainWin.CreateControls();
        mainWin.IDC_TITLE.setFont(new Font("Helvetica", Font.BOLD, 18));
        mainWin.IDC_STATUS.setText("");
        mainWin.IDC_VIEWMESSAGE.disable();
    }

    public void stop()
    {
        if (retriever != null)
        {
            retriever.stop();
            retriever = null;
        }
    }

    public boolean handleEvent(Event event)
    {
        boolean retval = false;
        if (event.target == mainWin.IDC_CHECKMSGS)
        {   // Only allow connect dialog to come up if not already retrieving.
            if (retriever == null || !retriever.isAlive())
                new ConnectDialog(this, connectData);
            retval = true;
        }
        else if (event.target == mainWin.IDC_VIEWMESSAGE && 
                mainWin.IDC_MESSAGES.countItems() > 0)
        {   // Bring up message viewer for current message.
            int sel = mainWin.IDC_MESSAGES.getSelectedIndex();
            if (sel >= 0)
                new ViewMessageDialog((String)(messages.elementAt(sel)));
            else
                mainWin.IDC_STATUS.setText("Please select a message to display.");
        }
        else if (event.arg instanceof ConnectData)
        {   // User selected OK from connect dialog, start retriever.
            mainWin.IDC_STATUS.setText("Retrieving new messages...");
            retriever = new MailRetriever(this);
            retval = true;
        }
        return retval;
    }

    public String getPopAccount()
    {   // Called by retriever to get POP account name.
        return connectData.acctName;
    }

    public String getPopPassword()
    {   // Called by retriever to get password for POP account.
        return connectData.acctPassword;
    }

    public String getPopServer()
    {   // Called by retriever to get the name of POP server.
        return connectData.popServer;
    }

    public synchronized void popStatus(String status)
    {   // Called by retriever to update app or applet on progress.
        mainWin.IDC_STATUS.setText(status);
    }

    public void popMessage(String message)
    {   // Called by retiever for each message retrieved.
        boolean fromFound = false, subjectFound = false;
        mainWin.IDC_VIEWMESSAGE.enable();

        // Parse message looking for whom the message is from and
        // the subject so we can format line in message listbox.
        StringTokenizer parse = new StringTokenizer(message, "\n");
        int lines = parse.countTokens();
        String from = "<Unknown From> ", subject = "<Unknown Subject> ";

        while (parse.hasMoreTokens())
        {
            String line = parse.nextToken();
            if (line.regionMatches(true, 0, "From", 0, 4) && !fromFound)
            {
                from = line + " ";
                fromFound = true;
            }
            else if (line.regionMatches(true, 0, "Subject", 0, 4) && !subjectFound)
            {
                subject = line + " ";
                subjectFound = true;
            }
        }
        // Add this message to list of messages.
        messages.addElement(message);

        mainWin.IDC_MESSAGES.addItem(from + subject + lines + " lines.");
    }
}

// Interface used between retriever and client applicaton/applet.
interface MailRetrieverClient
{
    public String getPopAccount();
    public String getPopPassword();
    public String getPopServer();
    public void popStatus(String status);
    public void popMessage(String message);
}

// Threaded POP mail retriever.
class MailRetriever extends Thread
{
    static final int POP_PORT = 110;
    Socket socket;
    PrintStream out;
    DataInputStream in;
    MailRetrieverClient client;
    String popResponse; // Holds last command/response from server.

    public MailRetriever(MailRetrieverClient client)
    {
        this.client = client;
        start();
    }

    public void run()
    {
        if (popConnect())
        {   // Connection established.
            client.popStatus("Logging on to POP server...");
            int messages = 0;
            if (popLogon())
            {   // Successfully logged on to server.
                client.popStatus("Checking for new messages...");
                messages = getWaitingMessageCount();

                for (int i = 1; i <= messages; i++)
                {
                    client.popStatus("Retrieving message " + i + " of " + messages + ".");
                    getMessage(i);
                }
    
                client.popStatus("Disconnecting from POP server...");
                popQuit();

                if (messages > 0)
                    client.popStatus("Retrieved " + messages + " new messages.");
                else
                    client.popStatus("No new messages retrieved.");
            }
            else
            {
                client.popStatus("Error logging on to POP server.");
                popQuit();
            }
        }
    }

    protected boolean popConnect()
    {
        boolean retval = true;
        String server = client.getPopServer();

        client.popStatus("Opening connection to " + server + "...");
        try
        {   // Make connection and setup buffered i/o streams.
            socket = new Socket(server, POP_PORT);
            in = new DataInputStream(new BufferedInputStream(socket.getInputStream()));
            out = new PrintStream(new BufferedOutputStream(socket.getOutputStream()), true);
            client.popStatus("Connection successfully established with " + server + ".");
        }
        catch (UnknownHostException e)
        {
            client.popStatus("Unable to resolve POP server's name.");
            retval = false;
        }
        catch (IOException e1)
        {
            client.popStatus("Error opening connection to POP server.");
            if (socket != null)
            {   // Close down socket.
                try
                    socket.close();
                catch (IOException e2) {}
            }
            retval = false;
        }
        return retval;
    }

    protected boolean waitForOk()
    {   // Waits for an OK response from server.
        boolean retval = true;
        try
        {
            do
            {
                popResponse = in.readLine();
            } while (!popResponse.startsWith("+OK"));
        }
        catch (Exception e)
            retval = false;
        return retval;
    }

    protected boolean popLogon()
    {   // Logs user on to server.
        boolean retval = false;
        if (waitForOk())
        {
            out.println("USER " + client.getPopAccount());

            if (waitForOk())
            {
                out.println("PASS " + client.getPopPassword());
                retval = waitForOk();
            }
        }
        return retval;
    }

    protected void popQuit()
    {   // Send QUIT command and close socket.
        out.println("QUIT");
        try
            socket.close();
        catch (IOException e) {}
    }

    protected int getWaitingMessageCount()
    {   // Get the number of messages waiting at server.
        int retval = 0;
        out.println("STAT");

        if (waitForOk())
        {
            StringTokenizer stats = new StringTokenizer(popResponse);
            stats.nextToken();
            retval = Integer.parseInt(stats.nextToken());
        }
        return retval;
    }

    protected boolean getMessage(int number)
    {   // Set the specified message number from server.
        boolean retval = false;
        out.println("RETR " + number);
        if (waitForOk())
        {
            String line, message = "";
            try
            {
                while ((line = in.readLine()) != null)
                {
                    if (line.equals("."))
                        break;
                    message += line;
                    message += "\n";    // Put newline back on.
                }
                // Pass message along to client.
                client.popMessage(message);
                // Delete message from server.
                out.println("DELE " + number);
                retval = waitForOk();
            }
            catch (IOException e)
                client.popStatus("Error retrieving message " + number + " from POP server.");
        }
        return retval;
    }
}

// Holds connection data and used to exchange data with connect dialog.
class ConnectData
{
    public String acctName;
    public String acctPassword;
    public String popServer;

    public ConnectData()
    {
        this("", "", "");
    }

    public ConnectData(String acctName, String acctPassword, String popServer)
    {
        this.acctName = acctName;
        this.acctPassword = acctPassword;
        this.popServer = popServer;
    }
}

// Frame to hold connect dialog.
class ConnectDialog extends Frame
{
    ConnectDlgRes res;
    ConnectData data;
    Applet parent;

    public ConnectDialog(Applet parent, ConnectData data)
    {
        super("Connect Parameters");
        this.data = data;
        this.parent = parent;
        setFont(new Font("Dialog", Font.PLAIN, 8));
        res = new ConnectDlgRes(this);
        res.CreateControls();
        res.IDC_ACCTNAME.setText(data.acctName);
        res.IDC_ACCTPASSWORD.setText(data.acctPassword);
        res.IDC_ACCTPASSWORD.setEchoCharacter('*');
        res.IDC_POPSERVER.setText(data.popServer);
        show();
    }

    public boolean handleEvent(Event event)
    {
        boolean retval = false;

        if (event.target instanceof Button)
        {
            retval = true;
            if (event.target == res.IDOK)
            {
                if (res.IDC_ACCTNAME.getText().length() == 0 || 
                        res.IDC_POPSERVER.getText().length() == 0)
                {
                    new MessageBox(this, "Account name and server are required.", "Error");
                    retval = false;
                }
                else
                {
                    data.acctName = res.IDC_ACCTNAME.getText();
                    data.acctPassword = res.IDC_ACCTPASSWORD.getText();
                    data.popServer = res.IDC_POPSERVER.getText();
                    Event evt = new Event(this, 0, data);
                    parent.postEvent(evt);
                }
            }
            if (retval)
                dispose();
        }
        else if (event.id == Event.WINDOW_DESTROY)
        {
            dispose();
            retval = true;
        }
        return retval;
    }
}

// Frame to hold message dialog.
class ViewMessageDialog extends Frame
{
    ViewMessageRes res;

    public ViewMessageDialog(String message)
    {
        super("View Message");
        setFont(new Font("Dialog", Font.PLAIN, 8));
        res = new ViewMessageRes(this);
        res.CreateControls();
        res.IDC_MESSAGE.setText(message);
        show();
    }

    public boolean handleEvent(Event event)
    {
        boolean retval = false;

        if (event.target == res.IDOK || event.id == Event.WINDOW_DESTROY)
        {
            dispose();
            retval = true;
        }

        return retval;
    }
}

// Main application frame used when program is run as
// an application rather than an applet.
class Ex18cApplicationFrame extends Frame
{
    MainWinRes resource;
    MainMenu menu;
    Applet owner;

	public Ex18cApplicationFrame(Applet owner)
	{
		super("Pop Client Application");
        this.owner = owner;

        menu = new MainMenu(this);
        menu.CreateMenu();

        setFont(new Font("Dialog", Font.PLAIN, 8));
        resource = new MainWinRes(this);
        resource.CreateControls();
        resource.IDC_TITLE.setFont(new Font("Helvetica", Font.BOLD, 18));
        show();
	}

	public boolean handleEvent(Event event)
	{
        if (event.target == menu.ID_FILEEXIT || event.id == Event.WINDOW_DESTROY)
        {
            dispose();
            System.exit(0);
            return true;
        }
        else
            return owner.handleEvent(event);
	}
}
